package tw.edu.scu.csim.scale;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.os.Bundle;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.view.MotionEvent;
import android.view.ScaleGestureDetector;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;

public class ScalePicture extends Activity {
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		/* ܦù */
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);

		setContentView(new ScalePictureView(this, null, 0));
	}

	public class ScalePictureView extends View {
		private Bitmap					mIcon;
		private float					mPosX;
		private float					mPosY;
		private float					mLastTouchX;
		private float					mLastTouchY;
		private ScaleGestureDetector	mScaleDetector;
		private float					mScaleFactor		= 1.f;
		private static final int		INVALID_POINTER_ID	= -1;
		private int						mActivePointerId	= INVALID_POINTER_ID;
		private int						screenWidth, screenHeight;					// ùe
		private float					mDegrees			= 0.f;
		private Matrix					matrix;

		public ScalePictureView(Context context, AttributeSet attrs, int defStyle) {
			super(context, attrs, defStyle);
			mIcon = BitmapFactory.decodeResource(context.getResources(), R.drawable.android);

			mScaleDetector = new ScaleGestureDetector(context, new ScaleListener());

			DisplayMetrics dm = new DisplayMetrics();
			getWindowManager().getDefaultDisplay().getMetrics(dm);
			screenWidth = dm.widthPixels;
			screenHeight = dm.heightPixels;
		}

		@Override
		public boolean onTouchEvent(MotionEvent ev) {
			mScaleDetector.onTouchEvent(ev);

			final int action = ev.getAction();
			switch (action & MotionEvent.ACTION_MASK) {
				case MotionEvent.ACTION_DOWN: {
					final float x = ev.getX();
					final float y = ev.getY();

					mLastTouchX = x;
					mLastTouchY = y;
					mActivePointerId = ev.getPointerId(0);
					break;
				}

				case MotionEvent.ACTION_MOVE: {
					final int pointerIndex = ev.findPointerIndex(mActivePointerId);
					final float x = ev.getX(pointerIndex);
					final float y = ev.getY(pointerIndex);

					if (!mScaleDetector.isInProgress()) {
						final float dx = x - mLastTouchX;
						final float dy = y - mLastTouchY;

						mPosX += dx;
						mPosY += dy;

						/* ʥX */
						if (mPosX < 0 || mPosX > screenWidth - mIcon.getWidth() * mScaleFactor || mPosY < 0 || mPosY > screenHeight - mIcon.getHeight() * mScaleFactor) {
							mPosX -= dx;
							mPosY -= dy;
						}

						invalidate();
					}

					mLastTouchX = x;
					mLastTouchY = y;
					break;
				}

				case MotionEvent.ACTION_UP: {
					mActivePointerId = INVALID_POINTER_ID;
					break;
				}

				case MotionEvent.ACTION_CANCEL: {
					mActivePointerId = INVALID_POINTER_ID;
					break;
				}

				case MotionEvent.ACTION_POINTER_UP: {
					final int pointerIndex = (ev.getAction() & MotionEvent.ACTION_POINTER_INDEX_MASK) >> MotionEvent.ACTION_POINTER_INDEX_SHIFT;
					final int pointerId = ev.getPointerId(pointerIndex);
					if (pointerId == mActivePointerId) {
						final int newPointerIndex = pointerIndex == 0 ? 1 : 0;
						mLastTouchX = ev.getX(newPointerIndex);
						mLastTouchY = ev.getY(newPointerIndex);
						mActivePointerId = ev.getPointerId(newPointerIndex);
					}
					break;
				}
			}
			return true;
		}

		@Override
		public void onDraw(Canvas canvas) {
			super.onDraw(canvas);

			canvas.save();
			canvas.translate(mPosX, mPosY);
			canvas.scale(mScaleFactor, mScaleFactor);
			canvas.drawBitmap(mIcon, 0, 0, null);
			// canvas.drawBitmap(mIcon, matrix, null);
			canvas.restore();
		}

		private class ScaleListener extends ScaleGestureDetector.SimpleOnScaleGestureListener {
			@Override
			public boolean onScale(ScaleGestureDetector detector) {
				// float lastScale = mScaleFactor;
				mScaleFactor *= detector.getScaleFactor();

				/*
				if (mScaleFactor < lastScale) {
					mDegrees -= 5;
				}
				else {
					mDegrees += 5;
				}
				matrix.setRotate(mDegrees, mIcon.getWidth() / 2, mIcon.getHeight() / 2);
				*/

				// YLjϹϤX
				if (mPosX + mIcon.getWidth() * mScaleFactor > screenWidth || mPosY + mIcon.getHeight() * mScaleFactor > screenHeight) {
					mScaleFactor /= detector.getScaleFactor();
				}

				mScaleFactor = Math.max(0.1f, Math.min(mScaleFactor, 3.0f)); // Yjp
				invalidate();
				return true;
			}
		}
	}
}
